@@ -27,6 +27,7 @@ import (
 	"github.com/influxdata/telegraf/plugins/parsers"
 	_ "github.com/influxdata/telegraf/plugins/parsers/all" // Blank import to have all parsers for testing
 	"github.com/influxdata/telegraf/plugins/parsers/json"
+	"github.com/influxdata/telegraf/plugins/processors"
 )
 
 func TestReadBinaryFile(t *testing.T) {
@@ -69,16 +70,16 @@ func TestConfig_LoadSingleInputWithEnvVars(t *testing.T) {
 		NamePass:  []string{"metricname1", "ip_192.168.1.1_name"},
 		FieldDrop: []string{"other", "stuff"},
 		FieldPass: []string{"some", "strings"},
-		TagDrop: []models.TagFilter{
+		TagDropFilters: []models.TagFilter{
 			{
 				Name:   "badtag",
-				Filter: []string{"othertag"},
+				Values: []string{"othertag"},
 			},
 		},
-		TagPass: []models.TagFilter{
+		TagPassFilters: []models.TagFilter{
 			{
 				Name:   "goodtag",
-				Filter: []string{"mytag"},
+				Values: []string{"mytag"},
 			},
 		},
 	}
@@ -99,7 +100,7 @@ func TestConfig_LoadSingleInputWithEnvVars(t *testing.T) {
 
 func TestConfig_LoadSingleInput(t *testing.T) {
 	c := NewConfig()
-	c.LoadConfig("./testdata/single_plugin.toml")
+	require.NoError(t, c.LoadConfig("./testdata/single_plugin.toml"))
 
 	input := inputs.Inputs["memcached"]().(*MockupInputPlugin)
 	input.Servers = []string{"localhost"}
@@ -109,16 +110,16 @@ func TestConfig_LoadSingleInput(t *testing.T) {
 		NamePass:  []string{"metricname1"},
 		FieldDrop: []string{"other", "stuff"},
 		FieldPass: []string{"some", "strings"},
-		TagDrop: []models.TagFilter{
+		TagDropFilters: []models.TagFilter{
 			{
 				Name:   "badtag",
-				Filter: []string{"othertag"},
+				Values: []string{"othertag"},
 			},
 		},
-		TagPass: []models.TagFilter{
+		TagPassFilters: []models.TagFilter{
 			{
 				Name:   "goodtag",
-				Filter: []string{"mytag"},
+				Values: []string{"mytag"},
 			},
 		},
 	}
@@ -154,16 +155,16 @@ func TestConfig_LoadDirectory(t *testing.T) {
 		NamePass:  []string{"metricname1"},
 		FieldDrop: []string{"other", "stuff"},
 		FieldPass: []string{"some", "strings"},
-		TagDrop: []models.TagFilter{
+		TagDropFilters: []models.TagFilter{
 			{
 				Name:   "badtag",
-				Filter: []string{"othertag"},
+				Values: []string{"othertag"},
 			},
 		},
-		TagPass: []models.TagFilter{
+		TagPassFilters: []models.TagFilter{
 			{
 				Name:   "goodtag",
-				Filter: []string{"mytag"},
+				Values: []string{"mytag"},
 			},
 		},
 	}
@@ -198,16 +199,16 @@ func TestConfig_LoadDirectory(t *testing.T) {
 		NamePass:  []string{"metricname1"},
 		FieldDrop: []string{"other", "stuff"},
 		FieldPass: []string{"some", "strings"},
-		TagDrop: []models.TagFilter{
+		TagDropFilters: []models.TagFilter{
 			{
 				Name:   "badtag",
-				Filter: []string{"othertag"},
+				Values: []string{"othertag"},
 			},
 		},
-		TagPass: []models.TagFilter{
+		TagPassFilters: []models.TagFilter{
 			{
 				Name:   "goodtag",
-				Filter: []string{"mytag"},
+				Values: []string{"mytag"},
 			},
 		},
 	}
@@ -264,6 +265,16 @@ func TestConfig_WrongCertPath(t *testing.T) {
 	require.Error(t, c.LoadConfig("./testdata/wrong_cert_path.toml"))
 }
 
+func TestConfig_DefaultParser(t *testing.T) {
+	c := NewConfig()
+	require.NoError(t, c.LoadConfig("./testdata/default_parser.toml"))
+}
+
+func TestConfig_DefaultExecParser(t *testing.T) {
+	c := NewConfig()
+	require.NoError(t, c.LoadConfig("./testdata/default_parser_exec.toml"))
+}
+
 func TestConfig_LoadSpecialTypes(t *testing.T) {
 	c := NewConfig()
 	require.NoError(t, c.LoadConfig("./testdata/special_types.toml"))
@@ -284,22 +295,82 @@ func TestConfig_LoadSpecialTypes(t *testing.T) {
 }
 
 func TestConfig_FieldNotDefined(t *testing.T) {
-	c := NewConfig()
-	err := c.LoadConfig("./testdata/invalid_field.toml")
-	require.Error(t, err, "invalid field name")
-	require.Equal(t, "Error loading config file ./testdata/invalid_field.toml: plugin inputs.http_listener_v2: line 1: configuration specified the fields [\"not_a_field\"], but they weren't used", err.Error())
+	tests := []struct {
+		name     string
+		filename string
+		expected string
+	}{
+		{
+			name:     "in input plugin without parser",
+			filename: "./testdata/invalid_field.toml",
+			expected: `line 1: configuration specified the fields ["not_a_field"], but they weren't used`,
+		},
+		{
+			name:     "in input plugin with parser",
+			filename: "./testdata/invalid_field_with_parser.toml",
+			expected: `line 1: configuration specified the fields ["not_a_field"], but they weren't used`,
+		},
+		{
+			name:     "in input plugin with parser func",
+			filename: "./testdata/invalid_field_with_parserfunc.toml",
+			expected: `line 1: configuration specified the fields ["not_a_field"], but they weren't used`,
+		},
+		{
+			name:     "in parser of input plugin",
+			filename: "./testdata/invalid_field_in_parser_table.toml",
+			expected: `line 1: configuration specified the fields ["not_a_field"], but they weren't used`,
+		},
+		{
+			name:     "in parser of input plugin with parser-func",
+			filename: "./testdata/invalid_field_in_parserfunc_table.toml",
+			expected: `line 1: configuration specified the fields ["not_a_field"], but they weren't used`,
+		},
+		{
+			name:     "in processor plugin without parser",
+			filename: "./testdata/invalid_field_processor.toml",
+			expected: `line 1: configuration specified the fields ["not_a_field"], but they weren't used`,
+		},
+		{
+			name:     "in processor plugin with parser",
+			filename: "./testdata/invalid_field_processor_with_parser.toml",
+			expected: `line 1: configuration specified the fields ["not_a_field"], but they weren't used`,
+		},
+		{
+			name:     "in processor plugin with parser func",
+			filename: "./testdata/invalid_field_processor_with_parserfunc.toml",
+			expected: `line 1: configuration specified the fields ["not_a_field"], but they weren't used`,
+		},
+		{
+			name:     "in parser of processor plugin",
+			filename: "./testdata/invalid_field_processor_in_parser_table.toml",
+			expected: `line 1: configuration specified the fields ["not_a_field"], but they weren't used`,
+		},
+		{
+			name:     "in parser of processor plugin with parser-func",
+			filename: "./testdata/invalid_field_processor_in_parserfunc_table.toml",
+			expected: `line 1: configuration specified the fields ["not_a_field"], but they weren't used`,
+		},
+	}
+
+	for _, tt := range tests {
+		t.Run(tt.name, func(t *testing.T) {
+			c := NewConfig()
+			err := c.LoadConfig(tt.filename)
+			require.ErrorContains(t, err, tt.expected)
+		})
+	}
 }
 
 func TestConfig_WrongFieldType(t *testing.T) {
 	c := NewConfig()
 	err := c.LoadConfig("./testdata/wrong_field_type.toml")
 	require.Error(t, err, "invalid field type")
-	require.Equal(t, "Error loading config file ./testdata/wrong_field_type.toml: error parsing http_listener_v2, line 2: (config.MockupInputPlugin.Port) cannot unmarshal TOML string into int", err.Error())
+	require.Equal(t, "error loading config file ./testdata/wrong_field_type.toml: error parsing http_listener_v2, line 2: (config.MockupInputPlugin.Port) cannot unmarshal TOML string into int", err.Error())
 
 	c = NewConfig()
 	err = c.LoadConfig("./testdata/wrong_field_type2.toml")
 	require.Error(t, err, "invalid field type2")
-	require.Equal(t, "Error loading config file ./testdata/wrong_field_type2.toml: error parsing http_listener_v2, line 2: (config.MockupInputPlugin.Methods) cannot unmarshal TOML string into []string", err.Error())
+	require.Equal(t, "error loading config file ./testdata/wrong_field_type2.toml: error parsing http_listener_v2, line 2: (config.MockupInputPlugin.Methods) cannot unmarshal TOML string into []string", err.Error())
 }
 
 func TestConfig_InlineTables(t *testing.T) {
@@ -332,7 +403,7 @@ func TestConfig_BadOrdering(t *testing.T) {
 	c := NewConfig()
 	err := c.LoadConfig("./testdata/non_slice_slice.toml")
 	require.Error(t, err, "bad ordering")
-	require.Equal(t, "Error loading config file ./testdata/non_slice_slice.toml: error parsing http array, line 4: cannot unmarshal TOML array into string (need slice)", err.Error())
+	require.Equal(t, "error loading config file ./testdata/non_slice_slice.toml: error parsing http array, line 4: cannot unmarshal TOML array into string (need slice)", err.Error())
 }
 
 func TestConfig_AzureMonitorNamespacePrefix(t *testing.T) {
@@ -358,7 +429,7 @@ func TestConfig_URLRetries3Fails(t *testing.T) {
 	}))
 	defer ts.Close()
 
-	expected := fmt.Sprintf("Error loading config file %s: Retry 3 of 3 failed to retrieve remote config: 404 Not Found", ts.URL)
+	expected := fmt.Sprintf("error loading config file %s: retry 3 of 3 failed to retrieve remote config: 404 Not Found", ts.URL)
 
 	c := NewConfig()
 	err := c.LoadConfig(ts.URL)
@@ -407,10 +478,10 @@ func TestConfig_URLLikeFileName(t *testing.T) {
 	require.Error(t, err)
 
 	if runtime.GOOS == "windows" {
-		// The error file not found error message is different on windows
-		require.Equal(t, "Error loading config file http:##www.example.com.conf: open http:##www.example.com.conf: The system cannot find the file specified.", err.Error())
+		// The error file not found error message is different on Windows
+		require.Equal(t, "error loading config file http:##www.example.com.conf: open http:##www.example.com.conf: The system cannot find the file specified.", err.Error())
 	} else {
-		require.Equal(t, "Error loading config file http:##www.example.com.conf: open http:##www.example.com.conf: no such file or directory", err.Error())
+		require.Equal(t, "error loading config file http:##www.example.com.conf: open http:##www.example.com.conf: no such file or directory", err.Error())
 	}
 }
 
@@ -654,16 +725,146 @@ func TestConfig_ParserInterfaceOldFormat(t *testing.T) {
 	}
 }
 
+func TestConfig_ProcessorsWithParsers(t *testing.T) {
+	formats := []string{
+		"collectd",
+		"csv",
+		"dropwizard",
+		"form_urlencoded",
+		"graphite",
+		"grok",
+		"influx",
+		"json",
+		"json_v2",
+		"logfmt",
+		"nagios",
+		"prometheus",
+		"prometheusremotewrite",
+		"value",
+		"wavefront",
+		"xml", "xpath_json", "xpath_msgpack", "xpath_protobuf",
+	}
+
+	c := NewConfig()
+	require.NoError(t, c.LoadConfig("./testdata/processors_with_parsers.toml"))
+	require.Len(t, c.Processors, len(formats))
+
+	override := map[string]struct {
+		param map[string]interface{}
+		mask  []string
+	}{
+		"csv": {
+			param: map[string]interface{}{
+				"HeaderRowCount": 42,
+			},
+			mask: []string{"TimeFunc", "ResetMode"},
+		},
+		"xpath_protobuf": {
+			param: map[string]interface{}{
+				"ProtobufMessageDef":  "testdata/addressbook.proto",
+				"ProtobufMessageType": "addressbook.AddressBook",
+			},
+		},
+	}
+
+	expected := make([]telegraf.Parser, 0, len(formats))
+	for _, format := range formats {
+		logger := models.NewLogger("parsers", format, "processors_with_parsers")
+
+		creator, found := parsers.Parsers[format]
+		require.Truef(t, found, "No parser for format %q", format)
+
+		parser := creator("parser_test")
+		if settings, found := override[format]; found {
+			s := reflect.Indirect(reflect.ValueOf(parser))
+			for key, value := range settings.param {
+				v := reflect.ValueOf(value)
+				s.FieldByName(key).Set(v)
+			}
+		}
+		models.SetLoggerOnPlugin(parser, logger)
+		if p, ok := parser.(telegraf.Initializer); ok {
+			require.NoError(t, p.Init())
+		}
+		expected = append(expected, parser)
+	}
+	require.Len(t, expected, len(formats))
+
+	actual := make([]interface{}, 0)
+	generated := make([]interface{}, 0)
+	for _, plugin := range c.Processors {
+		var processorIF telegraf.Processor
+		if p, ok := plugin.Processor.(unwrappable); ok {
+			processorIF = p.Unwrap()
+		} else {
+			processorIF = plugin.Processor.(telegraf.Processor)
+		}
+		require.NotNil(t, processorIF)
+
+		processor, ok := processorIF.(*MockupProcessorPluginParser)
+		require.True(t, ok)
+
+		// Get the parser set with 'SetParser()'
+		if p, ok := processor.Parser.(*models.RunningParser); ok {
+			actual = append(actual, p.Parser)
+		} else {
+			actual = append(actual, processor.Parser)
+		}
+		// Get the parser set with 'SetParserFunc()'
+		if processor.ParserFunc != nil {
+			g, err := processor.ParserFunc()
+			require.NoError(t, err)
+			if rp, ok := g.(*models.RunningParser); ok {
+				generated = append(generated, rp.Parser)
+			} else {
+				generated = append(generated, g)
+			}
+		} else {
+			generated = append(generated, nil)
+		}
+	}
+	require.Len(t, actual, len(formats))
+
+	for i, format := range formats {
+		// Determine the underlying type of the parser
+		stype := reflect.Indirect(reflect.ValueOf(expected[i])).Interface()
+		// Ignore all unexported fields and fields not relevant for functionality
+		options := []cmp.Option{
+			cmpopts.IgnoreUnexported(stype),
+			cmpopts.IgnoreTypes(sync.Mutex{}),
+			cmpopts.IgnoreInterfaces(struct{ telegraf.Logger }{}),
+		}
+		if settings, found := override[format]; found {
+			options = append(options, cmpopts.IgnoreFields(stype, settings.mask...))
+		}
+
+		// Do a manual comparision as require.EqualValues will also work on unexported fields
+		// that cannot be cleared or ignored.
+		diff := cmp.Diff(expected[i], actual[i], options...)
+		require.Emptyf(t, diff, "Difference in SetParser() for %q", format)
+		diff = cmp.Diff(expected[i], generated[i], options...)
+		require.Emptyf(t, diff, "Difference in SetParserFunc() for %q", format)
+	}
+}
+
 /*** Mockup INPUT plugin for (old) parser testing to avoid cyclic dependencies ***/
 type MockupInputPluginParserOld struct {
 	Parser     parsers.Parser
 	ParserFunc parsers.ParserFunc
 }
 
-func (m *MockupInputPluginParserOld) SampleConfig() string                  { return "Mockup old parser test plugin" }
-func (m *MockupInputPluginParserOld) Gather(acc telegraf.Accumulator) error { return nil }
-func (m *MockupInputPluginParserOld) SetParser(parser parsers.Parser)       { m.Parser = parser }
-func (m *MockupInputPluginParserOld) SetParserFunc(f parsers.ParserFunc)    { m.ParserFunc = f }
+func (m *MockupInputPluginParserOld) SampleConfig() string {
+	return "Mockup old parser test plugin"
+}
+func (m *MockupInputPluginParserOld) Gather(_ telegraf.Accumulator) error {
+	return nil
+}
+func (m *MockupInputPluginParserOld) SetParser(parser parsers.Parser) {
+	m.Parser = parser
+}
+func (m *MockupInputPluginParserOld) SetParserFunc(f parsers.ParserFunc) {
+	m.ParserFunc = f
+}
 
 /*** Mockup INPUT plugin for (new) parser testing to avoid cyclic dependencies ***/
 type MockupInputPluginParserNew struct {
@@ -671,10 +872,18 @@ type MockupInputPluginParserNew struct {
 	ParserFunc telegraf.ParserFunc
 }
 
-func (m *MockupInputPluginParserNew) SampleConfig() string                  { return "Mockup old parser test plugin" }
-func (m *MockupInputPluginParserNew) Gather(acc telegraf.Accumulator) error { return nil }
-func (m *MockupInputPluginParserNew) SetParser(parser telegraf.Parser)      { m.Parser = parser }
-func (m *MockupInputPluginParserNew) SetParserFunc(f telegraf.ParserFunc)   { m.ParserFunc = f }
+func (m *MockupInputPluginParserNew) SampleConfig() string {
+	return "Mockup old parser test plugin"
+}
+func (m *MockupInputPluginParserNew) Gather(_ telegraf.Accumulator) error {
+	return nil
+}
+func (m *MockupInputPluginParserNew) SetParser(parser telegraf.Parser) {
+	m.Parser = parser
+}
+func (m *MockupInputPluginParserNew) SetParserFunc(f telegraf.ParserFunc) {
+	m.ParserFunc = f
+}
 
 /*** Mockup INPUT plugin for testing to avoid cyclic dependencies ***/
 type MockupInputPlugin struct {
@@ -687,6 +896,7 @@ type MockupInputPlugin struct {
 	Paths        []string `toml:"paths"`
 	Port         int      `toml:"port"`
 	Command      string
+	Files        []string
 	PidFile      string
 	Log          telegraf.Logger `toml:"-"`
 	tls.ServerConfig
@@ -694,9 +904,140 @@ type MockupInputPlugin struct {
 	parser telegraf.Parser
 }
 
-func (m *MockupInputPlugin) SampleConfig() string                  { return "Mockup test input plugin" }
-func (m *MockupInputPlugin) Gather(acc telegraf.Accumulator) error { return nil }
-func (m *MockupInputPlugin) SetParser(parser telegraf.Parser)      { m.parser = parser }
+func (m *MockupInputPlugin) SampleConfig() string {
+	return "Mockup test input plugin"
+}
+func (m *MockupInputPlugin) Gather(_ telegraf.Accumulator) error {
+	return nil
+}
+func (m *MockupInputPlugin) SetParser(parser telegraf.Parser) {
+	m.parser = parser
+}
+
+/*** Mockup INPUT plugin with ParserFunc interface ***/
+type MockupInputPluginParserFunc struct {
+	parserFunc telegraf.ParserFunc
+}
+
+func (m *MockupInputPluginParserFunc) SampleConfig() string {
+	return "Mockup test input plugin"
+}
+func (m *MockupInputPluginParserFunc) Gather(_ telegraf.Accumulator) error {
+	return nil
+}
+func (m *MockupInputPluginParserFunc) SetParserFunc(pf telegraf.ParserFunc) {
+	m.parserFunc = pf
+}
+
+/*** Mockup INPUT plugin without ParserFunc interface ***/
+type MockupInputPluginParserOnly struct {
+	parser telegraf.Parser
+}
+
+func (m *MockupInputPluginParserOnly) SampleConfig() string {
+	return "Mockup test input plugin"
+}
+func (m *MockupInputPluginParserOnly) Gather(_ telegraf.Accumulator) error {
+	return nil
+}
+func (m *MockupInputPluginParserOnly) SetParser(p telegraf.Parser) {
+	m.parser = p
+}
+
+/*** Mockup PROCESSOR plugin for testing to avoid cyclic dependencies ***/
+type MockupProcessorPluginParser struct {
+	Parser     telegraf.Parser
+	ParserFunc telegraf.ParserFunc
+}
+
+func (m *MockupProcessorPluginParser) Start(_ telegraf.Accumulator) error {
+	return nil
+}
+func (m *MockupProcessorPluginParser) Stop() error {
+	return nil
+}
+func (m *MockupProcessorPluginParser) SampleConfig() string {
+	return "Mockup test processor plugin with parser"
+}
+func (m *MockupProcessorPluginParser) Apply(_ ...telegraf.Metric) []telegraf.Metric {
+	return nil
+}
+func (m *MockupProcessorPluginParser) Add(_ telegraf.Metric, _ telegraf.Accumulator) error {
+	return nil
+}
+func (m *MockupProcessorPluginParser) SetParser(parser telegraf.Parser) {
+	m.Parser = parser
+}
+func (m *MockupProcessorPluginParser) SetParserFunc(f telegraf.ParserFunc) {
+	m.ParserFunc = f
+}
+
+/*** Mockup PROCESSOR plugin without parser ***/
+type MockupProcessorPlugin struct{}
+
+func (m *MockupProcessorPlugin) Start(_ telegraf.Accumulator) error {
+	return nil
+}
+func (m *MockupProcessorPlugin) Stop() error {
+	return nil
+}
+func (m *MockupProcessorPlugin) SampleConfig() string {
+	return "Mockup test processor plugin with parser"
+}
+func (m *MockupProcessorPlugin) Apply(_ ...telegraf.Metric) []telegraf.Metric {
+	return nil
+}
+func (m *MockupProcessorPlugin) Add(_ telegraf.Metric, _ telegraf.Accumulator) error {
+	return nil
+}
+
+/*** Mockup PROCESSOR plugin with parser ***/
+type MockupProcessorPluginParserOnly struct {
+	Parser telegraf.Parser
+}
+
+func (m *MockupProcessorPluginParserOnly) Start(_ telegraf.Accumulator) error {
+	return nil
+}
+func (m *MockupProcessorPluginParserOnly) Stop() error {
+	return nil
+}
+func (m *MockupProcessorPluginParserOnly) SampleConfig() string {
+	return "Mockup test processor plugin with parser"
+}
+func (m *MockupProcessorPluginParserOnly) Apply(_ ...telegraf.Metric) []telegraf.Metric {
+	return nil
+}
+func (m *MockupProcessorPluginParserOnly) Add(_ telegraf.Metric, _ telegraf.Accumulator) error {
+	return nil
+}
+func (m *MockupProcessorPluginParserOnly) SetParser(parser telegraf.Parser) {
+	m.Parser = parser
+}
+
+/*** Mockup PROCESSOR plugin with parser-function ***/
+type MockupProcessorPluginParserFunc struct {
+	Parser telegraf.ParserFunc
+}
+
+func (m *MockupProcessorPluginParserFunc) Start(_ telegraf.Accumulator) error {
+	return nil
+}
+func (m *MockupProcessorPluginParserFunc) Stop() error {
+	return nil
+}
+func (m *MockupProcessorPluginParserFunc) SampleConfig() string {
+	return "Mockup test processor plugin with parser"
+}
+func (m *MockupProcessorPluginParserFunc) Apply(_ ...telegraf.Metric) []telegraf.Metric {
+	return nil
+}
+func (m *MockupProcessorPluginParserFunc) Add(_ telegraf.Metric, _ telegraf.Accumulator) error {
+	return nil
+}
+func (m *MockupProcessorPluginParserFunc) SetParserFunc(pf telegraf.ParserFunc) {
+	m.Parser = pf
+}
 
 /*** Mockup OUTPUT plugin for testing to avoid cyclic dependencies ***/
 type MockupOuputPlugin struct {
@@ -708,22 +1049,69 @@ type MockupOuputPlugin struct {
 	tls.ClientConfig
 }
 
-func (m *MockupOuputPlugin) Connect() error                        { return nil }
-func (m *MockupOuputPlugin) Close() error                          { return nil }
-func (m *MockupOuputPlugin) SampleConfig() string                  { return "Mockup test output plugin" }
-func (m *MockupOuputPlugin) Write(metrics []telegraf.Metric) error { return nil }
+func (m *MockupOuputPlugin) Connect() error {
+	return nil
+}
+func (m *MockupOuputPlugin) Close() error {
+	return nil
+}
+func (m *MockupOuputPlugin) SampleConfig() string {
+	return "Mockup test output plugin"
+}
+func (m *MockupOuputPlugin) Write(_ []telegraf.Metric) error {
+	return nil
+}
 
 // Register the mockup plugin on loading
 func init() {
 	// Register the mockup input plugin for the required names
-	inputs.Add("parser_test_new", func() telegraf.Input { return &MockupInputPluginParserNew{} })
-	inputs.Add("parser_test_old", func() telegraf.Input { return &MockupInputPluginParserOld{} })
-	inputs.Add("exec", func() telegraf.Input { return &MockupInputPlugin{Timeout: Duration(time.Second * 5)} })
-	inputs.Add("http_listener_v2", func() telegraf.Input { return &MockupInputPlugin{} })
-	inputs.Add("memcached", func() telegraf.Input { return &MockupInputPlugin{} })
-	inputs.Add("procstat", func() telegraf.Input { return &MockupInputPlugin{} })
+	inputs.Add("parser_test_new", func() telegraf.Input {
+		return &MockupInputPluginParserNew{}
+	})
+	inputs.Add("parser_test_old", func() telegraf.Input {
+		return &MockupInputPluginParserOld{}
+	})
+	inputs.Add("parser", func() telegraf.Input {
+		return &MockupInputPluginParserOnly{}
+	})
+	inputs.Add("parser_func", func() telegraf.Input {
+		return &MockupInputPluginParserFunc{}
+	})
+	inputs.Add("exec", func() telegraf.Input {
+		return &MockupInputPlugin{Timeout: Duration(time.Second * 5)}
+	})
+	inputs.Add("file", func() telegraf.Input {
+		return &MockupInputPlugin{}
+	})
+	inputs.Add("http_listener_v2", func() telegraf.Input {
+		return &MockupInputPlugin{}
+	})
+	inputs.Add("memcached", func() telegraf.Input {
+		return &MockupInputPlugin{}
+	})
+	inputs.Add("procstat", func() telegraf.Input {
+		return &MockupInputPlugin{}
+	})
+
+	// Register the mockup processor plugin for the required names
+	processors.Add("parser_test", func() telegraf.Processor {
+		return &MockupProcessorPluginParser{}
+	})
+	processors.Add("processor", func() telegraf.Processor {
+		return &MockupProcessorPlugin{}
+	})
+	processors.Add("processor_parser", func() telegraf.Processor {
+		return &MockupProcessorPluginParserOnly{}
+	})
+	processors.Add("processor_parserfunc", func() telegraf.Processor {
+		return &MockupProcessorPluginParserFunc{}
+	})
 
 	// Register the mockup output plugin for the required names
-	outputs.Add("azure_monitor", func() telegraf.Output { return &MockupOuputPlugin{NamespacePrefix: "Telegraf/"} })
-	outputs.Add("http", func() telegraf.Output { return &MockupOuputPlugin{} })
+	outputs.Add("azure_monitor", func() telegraf.Output {
+		return &MockupOuputPlugin{NamespacePrefix: "Telegraf/"}
+	})
+	outputs.Add("http", func() telegraf.Output {
+		return &MockupOuputPlugin{}
+	})
 }
