@@ -1,9 +1,11 @@
+//go:generate ../../../tools/readme_config_includer/generator
 package rabbitmq
 
 import (
+	_ "embed"
 	"encoding/json"
 	"fmt"
-	"io/ioutil"
+	"io"
 	"net/http"
 	"strconv"
 	"sync"
@@ -16,6 +18,9 @@ import (
 	"github.com/influxdata/telegraf/plugins/inputs"
 )
 
+//go:embed sample.conf
+var sampleConfig string
+
 // DefaultUsername will set a default value that corresponds to the default
 // value used by Rabbitmq
 const DefaultUsername = "guest"
@@ -36,7 +41,7 @@ const DefaultClientTimeout = 4
 // see the sample config for further details
 type RabbitMQ struct {
 	URL      string `toml:"url"`
-	Name     string `toml:"name"`
+	Name     string `toml:"name" deprecated:"1.3.0;use 'tags' instead"`
 	Username string `toml:"username"`
 	Password string `toml:"password"`
 	tls.ClientConfig
@@ -45,7 +50,7 @@ type RabbitMQ struct {
 	ClientTimeout         config.Duration `toml:"client_timeout"`
 
 	Nodes     []string `toml:"nodes"`
-	Queues    []string `toml:"queues"`
+	Queues    []string `toml:"queues" deprecated:"1.6.0;use 'queue_name_include' instead"`
 	Exchanges []string `toml:"exchanges"`
 
 	MetricInclude             []string `toml:"metric_include"`
@@ -138,6 +143,7 @@ type Queue struct {
 	IdleSince              string   `json:"idle_since"`
 	SlaveNodes             []string `json:"slave_nodes"`
 	SynchronisedSlaveNodes []string `json:"synchronised_slave_nodes"`
+	HeadMessageTimestamp   *int64   `json:"head_message_timestamp"`
 }
 
 // Node ...
@@ -269,65 +275,6 @@ var gatherFunctions = map[string]gatherFunc{
 	"queue":      gatherQueues,
 }
 
-var sampleConfig = `
-  ## Management Plugin url. (default: http://localhost:15672)
-  # url = "http://localhost:15672"
-  ## Tag added to rabbitmq_overview series; deprecated: use tags
-  # name = "rmq-server-1"
-  ## Credentials
-  # username = "guest"
-  # password = "guest"
-
-  ## Optional TLS Config
-  # tls_ca = "/etc/telegraf/ca.pem"
-  # tls_cert = "/etc/telegraf/cert.pem"
-  # tls_key = "/etc/telegraf/key.pem"
-  ## Use TLS but skip chain & host verification
-  # insecure_skip_verify = false
-
-  ## Optional request timeouts
-  ##
-  ## ResponseHeaderTimeout, if non-zero, specifies the amount of time to wait
-  ## for a server's response headers after fully writing the request.
-  # header_timeout = "3s"
-  ##
-  ## client_timeout specifies a time limit for requests made by this client.
-  ## Includes connection time, any redirects, and reading the response body.
-  # client_timeout = "4s"
-
-  ## A list of nodes to gather as the rabbitmq_node measurement. If not
-  ## specified, metrics for all nodes are gathered.
-  # nodes = ["rabbit@node1", "rabbit@node2"]
-
-  ## A list of queues to gather as the rabbitmq_queue measurement. If not
-  ## specified, metrics for all queues are gathered.
-  # queues = ["telegraf"]
-
-  ## A list of exchanges to gather as the rabbitmq_exchange measurement. If not
-  ## specified, metrics for all exchanges are gathered.
-  # exchanges = ["telegraf"]
-
-  ## Metrics to include and exclude. Globs accepted.
-  ## Note that an empty array for both will include all metrics
-  ## Currently the following metrics are supported: "exchange", "federation", "node", "overview", "queue"
-  # metric_include = []
-  # metric_exclude = []
-
-  ## Queues to include and exclude. Globs accepted.
-  ## Note that an empty array for both will include all queues
-  queue_name_include = []
-  queue_name_exclude = []
-
-  ## Federation upstreams include and exclude when gathering the rabbitmq_federation measurement.
-  ## If neither are specified, metrics for all federation upstreams are gathered.
-  ## Federation link metrics will only be gathered for queues and exchanges
-  ## whose non-federation metrics will be collected (e.g a queue excluded
-  ## by the 'queue_name_exclude' option will also be excluded from federation).
-  ## Globs accepted.
-  # federation_upstream_include = ["dataCentre-*"]
-  # federation_upstream_exclude = []
-`
-
 func boolToInt(b bool) int64 {
 	if b {
 		return 1
@@ -335,16 +282,10 @@ func boolToInt(b bool) int64 {
 	return 0
 }
 
-// SampleConfig ...
-func (r *RabbitMQ) SampleConfig() string {
+func (*RabbitMQ) SampleConfig() string {
 	return sampleConfig
 }
 
-// Description ...
-func (r *RabbitMQ) Description() string {
-	return "Reads metrics from RabbitMQ servers via the Management Plugin"
-}
-
 func (r *RabbitMQ) Init() error {
 	var err error
 
@@ -431,7 +372,7 @@ func (r *RabbitMQ) requestEndpoint(u string) ([]byte, error) {
 		return nil, fmt.Errorf("getting %q failed: %v %v", u, resp.StatusCode, http.StatusText(resp.StatusCode))
 	}
 
-	return ioutil.ReadAll(resp.Body)
+	return io.ReadAll(resp.Body)
 }
 
 func (r *RabbitMQ) requestJSON(u string, target interface{}) error {
@@ -464,7 +405,7 @@ func gatherOverview(r *RabbitMQ, acc telegraf.Accumulator) {
 		return
 	}
 
-	if overview.QueueTotals == nil || overview.ObjectTotals == nil || overview.MessageStats == nil || overview.Listeners == nil {
+	if overview.QueueTotals == nil || overview.ObjectTotals == nil || overview.MessageStats == nil {
 		acc.AddError(fmt.Errorf("Wrong answer from rabbitmq. Probably auth issue"))
 		return
 	}
@@ -644,36 +585,42 @@ func gatherQueues(r *RabbitMQ, acc telegraf.Accumulator) {
 			"auto_delete": strconv.FormatBool(queue.AutoDelete),
 		}
 
+		fields := map[string]interface{}{
+			// common information
+			"consumers":                queue.Consumers,
+			"consumer_utilisation":     queue.ConsumerUtilisation,
+			"idle_since":               queue.IdleSince,
+			"slave_nodes":              len(queue.SlaveNodes),
+			"synchronised_slave_nodes": len(queue.SynchronisedSlaveNodes),
+			"memory":                   queue.Memory,
+			// messages information
+			"message_bytes":             queue.MessageBytes,
+			"message_bytes_ready":       queue.MessageBytesReady,
+			"message_bytes_unacked":     queue.MessageBytesUnacknowledged,
+			"message_bytes_ram":         queue.MessageRAM,
+			"message_bytes_persist":     queue.MessagePersistent,
+			"messages":                  queue.Messages,
+			"messages_ready":            queue.MessagesReady,
+			"messages_unack":            queue.MessagesUnacknowledged,
+			"messages_ack":              queue.MessageStats.Ack,
+			"messages_ack_rate":         queue.MessageStats.AckDetails.Rate,
+			"messages_deliver":          queue.MessageStats.Deliver,
+			"messages_deliver_rate":     queue.MessageStats.DeliverDetails.Rate,
+			"messages_deliver_get":      queue.MessageStats.DeliverGet,
+			"messages_deliver_get_rate": queue.MessageStats.DeliverGetDetails.Rate,
+			"messages_publish":          queue.MessageStats.Publish,
+			"messages_publish_rate":     queue.MessageStats.PublishDetails.Rate,
+			"messages_redeliver":        queue.MessageStats.Redeliver,
+			"messages_redeliver_rate":   queue.MessageStats.RedeliverDetails.Rate,
+		}
+
+		if queue.HeadMessageTimestamp != nil {
+			fields["head_message_timestamp"] = *queue.HeadMessageTimestamp
+		}
+
 		acc.AddFields(
 			"rabbitmq_queue",
-			map[string]interface{}{
-				// common information
-				"consumers":                queue.Consumers,
-				"consumer_utilisation":     queue.ConsumerUtilisation,
-				"idle_since":               queue.IdleSince,
-				"slave_nodes":              len(queue.SlaveNodes),
-				"synchronised_slave_nodes": len(queue.SynchronisedSlaveNodes),
-				"memory":                   queue.Memory,
-				// messages information
-				"message_bytes":             queue.MessageBytes,
-				"message_bytes_ready":       queue.MessageBytesReady,
-				"message_bytes_unacked":     queue.MessageBytesUnacknowledged,
-				"message_bytes_ram":         queue.MessageRAM,
-				"message_bytes_persist":     queue.MessagePersistent,
-				"messages":                  queue.Messages,
-				"messages_ready":            queue.MessagesReady,
-				"messages_unack":            queue.MessagesUnacknowledged,
-				"messages_ack":              queue.MessageStats.Ack,
-				"messages_ack_rate":         queue.MessageStats.AckDetails.Rate,
-				"messages_deliver":          queue.MessageStats.Deliver,
-				"messages_deliver_rate":     queue.MessageStats.DeliverDetails.Rate,
-				"messages_deliver_get":      queue.MessageStats.DeliverGet,
-				"messages_deliver_get_rate": queue.MessageStats.DeliverGetDetails.Rate,
-				"messages_publish":          queue.MessageStats.Publish,
-				"messages_publish_rate":     queue.MessageStats.PublishDetails.Rate,
-				"messages_redeliver":        queue.MessageStats.Redeliver,
-				"messages_redeliver_rate":   queue.MessageStats.RedeliverDetails.Rate,
-			},
+			fields,
 			tags,
 		)
 	}
