@@ -2,18 +2,21 @@ package sql
 
 import (
 	"fmt"
+	"io"
 	"math/rand"
 	"os"
 	"path/filepath"
+	"strings"
 	"testing"
 	"time"
 
 	"github.com/docker/go-connections/nat"
+	"github.com/stretchr/testify/require"
+	"github.com/testcontainers/testcontainers-go/wait"
+
 	"github.com/influxdata/telegraf"
 	"github.com/influxdata/telegraf/metric"
 	"github.com/influxdata/telegraf/testutil"
-	"github.com/stretchr/testify/require"
-	"github.com/testcontainers/testcontainers-go/wait"
 )
 
 func TestSqlQuoteIntegration(t *testing.T) {
@@ -208,28 +211,36 @@ func TestMysqlIntegration(t *testing.T) {
 		testMetrics,
 	))
 
-	//dump the database
-	var rc int
-	rc, err = container.Exec([]string{
-		"bash",
-		"-c",
-		"mariadb-dump --user=" + username +
-			" --password=" + password +
-			" --compact --skip-opt " +
-			dbname +
-			" > /out/dump",
-	})
-	require.NoError(t, err)
-	require.Equal(t, 0, rc)
-	dumpfile := filepath.Join(outDir, "dump")
-	require.FileExists(t, dumpfile)
+	cases := []struct {
+		expectedFile string
+	}{
+		{"./testdata/mariadb/expected_metric_one.sql"},
+		{"./testdata/mariadb/expected_metric_two.sql"},
+		{"./testdata/mariadb/expected_metric_three.sql"},
+	}
+	for _, tc := range cases {
+		expected, err := os.ReadFile(tc.expectedFile)
+		require.NoError(t, err)
 
-	//compare the dump to what we expected
-	expected, err := os.ReadFile("testdata/mariadb/expected.sql")
-	require.NoError(t, err)
-	actual, err := os.ReadFile(dumpfile)
-	require.NoError(t, err)
-	require.Equal(t, string(expected), string(actual))
+		require.Eventually(t, func() bool {
+			rc, out, err := container.Exec([]string{
+				"bash",
+				"-c",
+				"mariadb-dump --user=" + username +
+					" --password=" + password +
+					" --compact --skip-opt " +
+					dbname,
+			})
+			require.NoError(t, err)
+			require.Equal(t, 0, rc)
+
+			bytes, err := io.ReadAll(out)
+			require.NoError(t, err)
+
+			fmt.Println(string(bytes))
+			return strings.Contains(string(bytes), string(expected))
+		}, 10*time.Second, 500*time.Millisecond, tc.expectedFile)
+	}
 }
 
 func TestPostgresIntegration(t *testing.T) {
@@ -262,7 +273,7 @@ func TestPostgresIntegration(t *testing.T) {
 		ExposedPorts: []string{servicePort},
 		WaitingFor: wait.ForAll(
 			wait.ForListeningPort(nat.Port(servicePort)),
-			wait.ForLog("database system is ready to accept connections"),
+			wait.ForLog("database system is ready to accept connections").WithOccurrence(2),
 		),
 	}
 	err = container.Start()
@@ -289,38 +300,35 @@ func TestPostgresIntegration(t *testing.T) {
 		testMetrics,
 	))
 
-	//dump the database
-	//psql -u postgres
-	var rc int
-	rc, err = container.Exec([]string{
-		"bash",
-		"-c",
-		"pg_dump" +
-			" --username=" + username +
-			//" --password=" + password +
-			//			" --compact --skip-opt " +
-			" --no-comments" +
-			//" --data-only" +
-			" " + dbname +
-			// pg_dump's output has comments that include build info
-			// of postgres and pg_dump. The build info changes with
-			// each release. To prevent these changes from causing the
-			// test to fail, we strip out comments. Also strip out
-			// blank lines.
-			"|grep -E -v '(^--|^$)'" +
-			" > /out/dump 2>&1",
-	})
+	expected, err := os.ReadFile("./testdata/postgres/expected.sql")
 	require.NoError(t, err)
-	require.Equal(t, 0, rc)
-	dumpfile := filepath.Join(outDir, "dump")
-	require.FileExists(t, dumpfile)
 
-	//compare the dump to what we expected
-	expected, err := os.ReadFile("testdata/postgres/expected.sql")
-	require.NoError(t, err)
-	actual, err := os.ReadFile(dumpfile)
-	require.NoError(t, err)
-	require.Equal(t, string(expected), string(actual))
+	require.Eventually(t, func() bool {
+		rc, out, err := container.Exec([]string{
+			"bash",
+			"-c",
+			"pg_dump" +
+				" --username=" + username +
+				//" --password=" + password +
+				//			" --compact --skip-opt " +
+				" --no-comments" +
+				//" --data-only" +
+				" " + dbname +
+				// pg_dump's output has comments that include build info
+				// of postgres and pg_dump. The build info changes with
+				// each release. To prevent these changes from causing the
+				// test to fail, we strip out comments. Also strip out
+				// blank lines.
+				"|grep -E -v '(^--|^$)'",
+		})
+		require.NoError(t, err)
+		require.Equal(t, 0, rc)
+
+		bytes, err := io.ReadAll(out)
+		require.NoError(t, err)
+
+		return strings.Contains(string(bytes), string(expected))
+	}, 5*time.Second, 500*time.Millisecond)
 }
 
 func TestClickHouseIntegration(t *testing.T) {
@@ -351,7 +359,7 @@ func TestClickHouseIntegration(t *testing.T) {
 		WaitingFor: wait.ForAll(
 			wait.NewHTTPStrategy("/").WithPort(nat.Port("8123")),
 			wait.ForListeningPort(nat.Port(servicePort)),
-			wait.ForLog("Saved preprocessed configuration to '/var/lib/clickhouse/preprocessed_configs/users.xml'"),
+			wait.ForLog("Saved preprocessed configuration to '/var/lib/clickhouse/preprocessed_configs/users.xml'").WithOccurrence(2),
 		),
 	}
 	err = container.Start()
@@ -378,35 +386,34 @@ func TestClickHouseIntegration(t *testing.T) {
 	p.Convert.ConversionStyle = "literal"
 
 	require.NoError(t, p.Connect())
-
 	require.NoError(t, p.Write(testMetrics))
 
-	// dump the database
-	var rc int
-	for _, testMetric := range testMetrics {
-		rc, err = container.Exec([]string{
-			"bash",
-			"-c",
-			"clickhouse-client" +
-				" --user=" + username +
-				" --database=" + dbname +
-				" --format=TabSeparatedRaw" +
-				" --multiquery --query=" +
-				"\"SELECT * FROM \\\"" + testMetric.Name() + "\\\";" +
-				"SHOW CREATE TABLE \\\"" + testMetric.Name() + "\\\"\"" +
-				" >> /out/dump 2>&1",
-		})
-		require.NoError(t, err)
-		require.Equal(t, 0, rc)
+	cases := []struct {
+		table    string
+		expected string
+	}{
+		{"metric_one", "`float64_one` Float64"},
+		{"metric_two", "`string_one` String"},
+		{"metric three", "`string two` String"},
+	}
+	for _, tc := range cases {
+		require.Eventually(t, func() bool {
+			var out io.Reader
+			_, out, err = container.Exec([]string{
+				"bash",
+				"-c",
+				"clickhouse-client" +
+					" --user=" + username +
+					" --database=" + dbname +
+					" --format=TabSeparatedRaw" +
+					" --multiquery --query=" +
+					"\"SELECT * FROM \\\"" + tc.table + "\\\";" +
+					"SHOW CREATE TABLE \\\"" + tc.table + "\\\"\"",
+			})
+			require.NoError(t, err)
+			bytes, err := io.ReadAll(out)
+			require.NoError(t, err)
+			return strings.Contains(string(bytes), tc.expected)
+		}, 5*time.Second, 500*time.Millisecond)
 	}
-
-	dumpfile := filepath.Join(outDir, "dump")
-	require.FileExists(t, dumpfile)
-
-	//compare the dump to what we expected
-	expected, err := os.ReadFile("testdata/clickhouse/expected.txt")
-	require.NoError(t, err)
-	actual, err := os.ReadFile(dumpfile)
-	require.NoError(t, err)
-	require.Equal(t, string(expected), string(actual))
 }
