@@ -2,22 +2,27 @@ package json_v2
 
 import (
 	"fmt"
+	"io"
 	"strconv"
 	"strings"
 	"time"
 
+	"github.com/dimchansky/utfbom"
+	"github.com/tidwall/gjson"
+
 	"github.com/influxdata/telegraf"
 	"github.com/influxdata/telegraf/internal"
 	"github.com/influxdata/telegraf/metric"
-	"github.com/tidwall/gjson"
+	"github.com/influxdata/telegraf/plugins/parsers"
+	"github.com/influxdata/telegraf/plugins/parsers/temporary/json_v2"
 )
 
 // Parser adheres to the parser interface, contains the parser configuration, and data required to parse JSON
 type Parser struct {
-	// These struct fields are common for a parser
-	Configs     []Config
-	DefaultTags map[string]string
-	Log         telegraf.Logger
+	Configs           []json_v2.Config  `toml:"json_v2"`
+	DefaultMetricName string            `toml:"-"`
+	DefaultTags       map[string]string `toml:"-"`
+	Log               telegraf.Logger   `toml:"-"`
 
 	// **** The struct fields bellow this comment are used for processing indvidual configs ****
 
@@ -30,48 +35,13 @@ type Parser struct {
 	// iterateObjects dictates if ExpandArray function will handle objects
 	iterateObjects bool
 	// objectConfig contains the config for an object, some info is needed while iterating over the gjson results
-	objectConfig JSONObject
+	objectConfig json_v2.Object
 }
 
 type PathResult struct {
 	result gjson.Result
 	tag    bool
-	DataSet
-}
-
-type Config struct {
-	MeasurementName     string `toml:"measurement_name"`      // OPTIONAL
-	MeasurementNamePath string `toml:"measurement_name_path"` // OPTIONAL
-	TimestampPath       string `toml:"timestamp_path"`        // OPTIONAL
-	TimestampFormat     string `toml:"timestamp_format"`      // OPTIONAL, but REQUIRED when timestamp_path is defined
-	TimestampTimezone   string `toml:"timestamp_timezone"`    // OPTIONAL, but REQUIRES timestamp_path
-
-	Fields      []DataSet
-	Tags        []DataSet
-	JSONObjects []JSONObject
-}
-
-type DataSet struct {
-	Path     string `toml:"path"` // REQUIRED
-	Type     string `toml:"type"` // OPTIONAL, can't be set for tags they will always be a string
-	Rename   string `toml:"rename"`
-	Optional bool   `toml:"optional"` // Will suppress errors if there isn't a match with Path
-}
-
-type JSONObject struct {
-	Path               string            `toml:"path"`     // REQUIRED
-	Optional           bool              `toml:"optional"` // Will suppress errors if there isn't a match with Path
-	TimestampKey       string            `toml:"timestamp_key"`
-	TimestampFormat    string            `toml:"timestamp_format"`   // OPTIONAL, but REQUIRED when timestamp_path is defined
-	TimestampTimezone  string            `toml:"timestamp_timezone"` // OPTIONAL, but REQUIRES timestamp_path
-	Renames            map[string]string `toml:"renames"`
-	Fields             map[string]string `toml:"fields"`
-	Tags               []string          `toml:"tags"`
-	IncludedKeys       []string          `toml:"included_keys"`
-	ExcludedKeys       []string          `toml:"excluded_keys"`
-	DisablePrependKeys bool              `toml:"disable_prepend_keys"`
-	FieldPaths         []DataSet
-	TagPaths           []DataSet
+	json_v2.DataSet
 }
 
 type MetricNode struct {
@@ -90,7 +60,24 @@ type MetricNode struct {
 	gjson.Result
 }
 
+func (p *Parser) Init() error {
+	// Propagate the default metric name to the configs in case it is not set there
+	for i, cfg := range p.Configs {
+		if cfg.MeasurementName == "" {
+			p.Configs[i].MeasurementName = p.DefaultMetricName
+		}
+	}
+	return nil
+}
+
 func (p *Parser) Parse(input []byte) ([]telegraf.Metric, error) {
+	reader := strings.NewReader(string(input))
+	body, _ := utfbom.Skip(reader)
+	input, err := io.ReadAll(body)
+	if err != nil {
+		return nil, fmt.Errorf("unable to read body after BOM removal: %v", err)
+	}
+
 	// Only valid JSON is supported
 	if !gjson.Valid(string(input)) {
 		return nil, fmt.Errorf("invalid JSON provided, unable to parse")
@@ -168,7 +155,7 @@ func (p *Parser) Parse(input []byte) ([]telegraf.Metric, error) {
 // processMetric will iterate over all 'field' or 'tag' configs and create metrics for each
 // A field/tag can either be a single value or an array of values, each resulting in its own metric
 // For multiple configs, a set of metrics is created from the cartesian product of each separate config
-func (p *Parser) processMetric(input []byte, data []DataSet, tag bool, timestamp time.Time) ([]telegraf.Metric, error) {
+func (p *Parser) processMetric(input []byte, data []json_v2.DataSet, tag bool, timestamp time.Time) ([]telegraf.Metric, error) {
 	if len(data) == 0 {
 		return nil, nil
 	}
@@ -410,7 +397,7 @@ func (p *Parser) existsInpathResults(index int) *PathResult {
 }
 
 // processObjects will iterate over all 'object' configs and create metrics for each
-func (p *Parser) processObjects(input []byte, objects []JSONObject, timestamp time.Time) ([]telegraf.Metric, error) {
+func (p *Parser) processObjects(input []byte, objects []json_v2.Object, timestamp time.Time) ([]telegraf.Metric, error) {
 	p.iterateObjects = true
 	var t []telegraf.Metric
 	for _, c := range objects {
@@ -678,3 +665,26 @@ func (p *Parser) checkResult(result gjson.Result, path string, optional bool) (b
 
 	return false, nil
 }
+
+func init() {
+	// Register all variants
+	parsers.Add("json_v2",
+		func(defaultMetricName string) telegraf.Parser {
+			return &Parser{DefaultMetricName: defaultMetricName}
+		},
+	)
+}
+
+// InitFromConfig is a compatibility function to construct the parser the old way
+func (p *Parser) InitFromConfig(config *parsers.Config) error {
+	p.DefaultMetricName = config.MetricName
+	p.DefaultTags = config.DefaultTags
+
+	// Convert the config formats which is a one-to-one copy
+	if len(config.JSONV2Config) > 0 {
+		p.Configs = make([]json_v2.Config, 0, len(config.JSONV2Config))
+		p.Configs = append(p.Configs, config.JSONV2Config...)
+	}
+
+	return p.Init()
+}
