@@ -9,91 +9,78 @@ import (
 	"testing"
 	"time"
 
-	"github.com/influxdata/telegraf"
-	"github.com/influxdata/telegraf/config"
-	"github.com/influxdata/telegraf/plugins/parsers"
-	"github.com/influxdata/telegraf/testutil"
 	"github.com/stretchr/testify/require"
 	starlarktime "go.starlark.net/lib/time"
 	"go.starlark.net/starlark"
 	"go.starlark.net/starlarkstruct"
+
+	"github.com/influxdata/telegraf"
+	"github.com/influxdata/telegraf/config"
+	common "github.com/influxdata/telegraf/plugins/common/starlark"
+	"github.com/influxdata/telegraf/plugins/parsers/influx"
+	"github.com/influxdata/telegraf/testutil"
 )
 
 // Tests for runtime errors in the processors Init function.
 func TestInitError(t *testing.T) {
 	tests := []struct {
-		name   string
-		plugin *Starlark
+		name      string
+		constants map[string]interface{}
+		plugin    *Starlark
 	}{
 		{
-			name: "source must define apply",
-			plugin: &Starlark{
-				Source:           "",
-				Log:              testutil.Logger{},
-				starlarkLoadFunc: testLoadFunc,
-			},
+			name:   "source must define apply",
+			plugin: newStarlarkFromSource(""),
 		},
 		{
 			name: "apply must be a function",
-			plugin: &Starlark{
-				Source: `
+			plugin: newStarlarkFromSource(`
 apply = 42
-`,
-				Log:              testutil.Logger{},
-				starlarkLoadFunc: testLoadFunc,
-			},
+`),
 		},
 		{
 			name: "apply function must take one arg",
-			plugin: &Starlark{
-				Source: `
+			plugin: newStarlarkFromSource(`
 def apply():
 	pass
-`,
-				Log:              testutil.Logger{},
-				starlarkLoadFunc: testLoadFunc,
-			},
+`),
 		},
 		{
 			name: "package scope must have valid syntax",
-			plugin: &Starlark{
-				Source: `
+			plugin: newStarlarkFromSource(`
 for
-`,
-				Log:              testutil.Logger{},
-				starlarkLoadFunc: testLoadFunc,
-			},
+`),
 		},
 		{
-			name: "no source no script",
-			plugin: &Starlark{
-				Log:              testutil.Logger{},
-				starlarkLoadFunc: testLoadFunc,
-			},
+			name:   "no source no script",
+			plugin: newStarlarkNoScript(),
 		},
 		{
 			name: "source and script",
-			plugin: &Starlark{
-				Source: `
+			plugin: newStarlarkFromSource(`
 def apply():
 	pass
-`,
-				Script:           "testdata/ratio.star",
-				Log:              testutil.Logger{},
-				starlarkLoadFunc: testLoadFunc,
-			},
+`),
 		},
 		{
-			name: "script file not found",
-			plugin: &Starlark{
-				Script:           "testdata/file_not_found.star",
-				Log:              testutil.Logger{},
-				starlarkLoadFunc: testLoadFunc,
+			name:   "script file not found",
+			plugin: newStarlarkFromScript("testdata/file_not_found.star"),
+		},
+		{
+			name: "source and script",
+			plugin: newStarlarkFromSource(`
+def apply(metric):
+	metric.fields["p1"] = unsupported_type
+	return metric
+`),
+			constants: map[string]interface{}{
+				"unsupported_type": time.Now(),
 			},
 		},
 	}
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
+			tt.plugin.Constants = tt.constants
 			err := tt.plugin.Init()
 			require.Error(t, err)
 		})
@@ -227,11 +214,7 @@ def apply(metric):
 
 	for _, tt := range applyTests {
 		t.Run(tt.name, func(t *testing.T) {
-			plugin := &Starlark{
-				Source:           tt.source,
-				Log:              testutil.Logger{},
-				starlarkLoadFunc: testLoadFunc,
-			}
+			plugin := newStarlarkFromSource(tt.source)
 			err := plugin.Init()
 			require.NoError(t, err)
 
@@ -2545,7 +2528,6 @@ def apply(metric):
 					2:   "two",
 					"3": "three",
 				},
-				"unsupported_type": time.Now(),
 			},
 			input: []telegraf.Metric{
 				testutil.MustMetric("cpu",
@@ -2575,12 +2557,8 @@ def apply(metric):
 
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
-			plugin := &Starlark{
-				Source:           tt.source,
-				Log:              testutil.Logger{},
-				Constants:        tt.constants,
-				starlarkLoadFunc: testLoadFunc,
-			}
+			plugin := newStarlarkFromSource(tt.source)
+			plugin.Constants = tt.constants
 			err := plugin.Init()
 			require.NoError(t, err)
 
@@ -2637,7 +2615,6 @@ def apply(metric):
 	debug_mode = true
 	supported_values = ["2", "3"]
 	supported_entries = { "2" = "two", "3" = "three" }
-	unsupported_type = 2009-06-12
            `,
 			input: []telegraf.Metric{
 				testutil.MustMetric("cpu",
@@ -2698,11 +2675,11 @@ func buildPlugin(configContent string) (*Starlark, error) {
 		return nil, err
 	}
 	if len(c.Processors) != 1 {
-		return nil, errors.New("Only one processor was expected")
+		return nil, errors.New("only one processor was expected")
 	}
 	plugin, ok := (c.Processors[0].Processor).(*Starlark)
 	if !ok {
-		return nil, errors.New("Only a Starlark processor was expected")
+		return nil, errors.New("only a Starlark processor was expected")
 	}
 	plugin.Log = testutil.Logger{}
 	return plugin, nil
@@ -2717,12 +2694,8 @@ func TestScript(t *testing.T) {
 		expectedErrorStr string
 	}{
 		{
-			name: "rename",
-			plugin: &Starlark{
-				Script:           "testdata/rename.star",
-				Log:              testutil.Logger{},
-				starlarkLoadFunc: testLoadFunc,
-			},
+			name:   "rename",
+			plugin: newStarlarkFromScript("testdata/rename.star"),
 			input: []telegraf.Metric{
 				testutil.MustMetric("cpu",
 					map[string]string{
@@ -2745,12 +2718,8 @@ func TestScript(t *testing.T) {
 			},
 		},
 		{
-			name: "drop fields by type",
-			plugin: &Starlark{
-				Script:           "testdata/drop_string_fields.star",
-				Log:              testutil.Logger{},
-				starlarkLoadFunc: testLoadFunc,
-			},
+			name:   "drop fields by type",
+			plugin: newStarlarkFromScript("testdata/drop_string_fields.star"),
 			input: []telegraf.Metric{
 				testutil.MustMetric("device",
 					map[string]string{},
@@ -2777,12 +2746,8 @@ func TestScript(t *testing.T) {
 			},
 		},
 		{
-			name: "drop fields with unexpected type",
-			plugin: &Starlark{
-				Script:           "testdata/drop_fields_with_unexpected_type.star",
-				Log:              testutil.Logger{},
-				starlarkLoadFunc: testLoadFunc,
-			},
+			name:   "drop fields with unexpected type",
+			plugin: newStarlarkFromScript("testdata/drop_fields_with_unexpected_type.star"),
 			input: []telegraf.Metric{
 				testutil.MustMetric("device",
 					map[string]string{},
@@ -2812,12 +2777,8 @@ func TestScript(t *testing.T) {
 			},
 		},
 		{
-			name: "scale",
-			plugin: &Starlark{
-				Script:           "testdata/scale.star",
-				Log:              testutil.Logger{},
-				starlarkLoadFunc: testLoadFunc,
-			},
+			name:   "scale",
+			plugin: newStarlarkFromScript("testdata/scale.star"),
 			input: []telegraf.Metric{
 				testutil.MustMetric("cpu",
 					map[string]string{},
@@ -2834,12 +2795,8 @@ func TestScript(t *testing.T) {
 			},
 		},
 		{
-			name: "ratio",
-			plugin: &Starlark{
-				Script:           "testdata/ratio.star",
-				Log:              testutil.Logger{},
-				starlarkLoadFunc: testLoadFunc,
-			},
+			name:   "ratio",
+			plugin: newStarlarkFromScript("testdata/ratio.star"),
 			input: []telegraf.Metric{
 				testutil.MustMetric("mem",
 					map[string]string{},
@@ -2863,12 +2820,8 @@ func TestScript(t *testing.T) {
 			},
 		},
 		{
-			name: "logging",
-			plugin: &Starlark{
-				Script:           "testdata/logging.star",
-				Log:              testutil.Logger{},
-				starlarkLoadFunc: testLoadFunc,
-			},
+			name:   "logging",
+			plugin: newStarlarkFromScript("testdata/logging.star"),
 			input: []telegraf.Metric{
 				testutil.MustMetric("log",
 					map[string]string{},
@@ -2889,12 +2842,8 @@ func TestScript(t *testing.T) {
 			},
 		},
 		{
-			name: "multiple_metrics",
-			plugin: &Starlark{
-				Script:           "testdata/multiple_metrics.star",
-				Log:              testutil.Logger{},
-				starlarkLoadFunc: testLoadFunc,
-			},
+			name:   "multiple_metrics",
+			plugin: newStarlarkFromScript("testdata/multiple_metrics.star"),
 			input: []telegraf.Metric{
 				testutil.MustMetric("mm",
 					map[string]string{},
@@ -2922,12 +2871,8 @@ func TestScript(t *testing.T) {
 			},
 		},
 		{
-			name: "multiple_metrics_with_json",
-			plugin: &Starlark{
-				Script:           "testdata/multiple_metrics_with_json.star",
-				Log:              testutil.Logger{},
-				starlarkLoadFunc: testLoadFunc,
-			},
+			name:   "multiple_metrics_with_json",
+			plugin: newStarlarkFromScript("testdata/multiple_metrics_with_json.star"),
 			input: []telegraf.Metric{
 				testutil.MustMetric("json",
 					map[string]string{},
@@ -2955,12 +2900,8 @@ func TestScript(t *testing.T) {
 			},
 		},
 		{
-			name: "fail",
-			plugin: &Starlark{
-				Script:           "testdata/fail.star",
-				Log:              testutil.Logger{},
-				starlarkLoadFunc: testLoadFunc,
-			},
+			name:   "fail",
+			plugin: newStarlarkFromScript("testdata/fail.star"),
 			input: []telegraf.Metric{
 				testutil.MustMetric("fail",
 					map[string]string{},
@@ -3242,15 +3183,73 @@ def apply(metric):
 				),
 			},
 		},
+		{
+			name: "concatenate 2 tags",
+			source: `
+def apply(metric):
+	metric.tags["result"] = '_'.join(metric.tags.values())
+	return metric
+`,
+			input: []telegraf.Metric{
+				testutil.MustMetric("cpu",
+					map[string]string{
+						"tag_1": "a",
+						"tag_2": "b",
+					},
+					map[string]interface{}{"value": 42},
+					time.Unix(0, 0),
+				),
+			},
+		},
+		{
+			name: "concatenate 4 tags",
+			source: `
+def apply(metric):
+	metric.tags["result"] = '_'.join(metric.tags.values())
+	return metric
+`,
+			input: []telegraf.Metric{
+				testutil.MustMetric("cpu",
+					map[string]string{
+						"tag_1": "a",
+						"tag_2": "b",
+						"tag_3": "c",
+						"tag_4": "d",
+					},
+					map[string]interface{}{"value": 42},
+					time.Unix(0, 0),
+				),
+			},
+		},
+		{
+			name: "concatenate 8 tags",
+			source: `
+def apply(metric):
+	metric.tags["result"] = '_'.join(metric.tags.values())
+	return metric
+`,
+			input: []telegraf.Metric{
+				testutil.MustMetric("cpu",
+					map[string]string{
+						"tag_1": "a",
+						"tag_2": "b",
+						"tag_3": "c",
+						"tag_4": "d",
+						"tag_5": "e",
+						"tag_6": "f",
+						"tag_7": "g",
+						"tag_8": "h",
+					},
+					map[string]interface{}{"value": 42},
+					time.Unix(0, 0),
+				),
+			},
+		},
 	}
 
 	for _, tt := range tests {
 		b.Run(tt.name, func(b *testing.B) {
-			plugin := &Starlark{
-				Source:           tt.source,
-				Log:              testutil.Logger{},
-				starlarkLoadFunc: testLoadFunc,
-			}
+			plugin := newStarlarkFromSource(tt.source)
 
 			err := plugin.Init()
 			require.NoError(b, err)
@@ -3263,7 +3262,8 @@ def apply(metric):
 			b.ResetTimer()
 			for n := 0; n < b.N; n++ {
 				for _, m := range tt.input {
-					plugin.Add(m, &acc)
+					err = plugin.Add(m, &acc)
+					require.NoError(b, err)
 				}
 			}
 
@@ -3277,7 +3277,7 @@ func TestAllScriptTestData(t *testing.T) {
 	// can be run from multiple folders
 	paths := []string{"testdata", "plugins/processors/starlark/testdata"}
 	for _, testdataPath := range paths {
-		filepath.Walk(testdataPath, func(path string, info os.FileInfo, err error) error {
+		err := filepath.Walk(testdataPath, func(path string, info os.FileInfo, err error) error {
 			if info == nil || info.IsDir() {
 				return nil
 			}
@@ -3292,11 +3292,7 @@ func TestAllScriptTestData(t *testing.T) {
 				if expectedErrorStr == "" {
 					outputMetrics = parseMetricsFrom(t, lines, "Example Output:")
 				}
-				plugin := &Starlark{
-					Script:           fn,
-					Log:              testutil.Logger{},
-					starlarkLoadFunc: testLoadFunc,
-				}
+				plugin := newStarlarkFromScript(fn)
 				require.NoError(t, plugin.Init())
 
 				acc := &testutil.Accumulator{}
@@ -3320,13 +3316,15 @@ func TestAllScriptTestData(t *testing.T) {
 			})
 			return nil
 		})
+		require.NoError(t, err)
 	}
 }
 
-var parser, _ = parsers.NewInfluxParser() // literally never returns errors.
-
 // parses metric lines out of line protocol following a header, with a trailing blank line
 func parseMetricsFrom(t *testing.T, lines []string, header string) (metrics []telegraf.Metric) {
+	parser := &influx.Parser{}
+	require.NoError(t, parser.Init())
+
 	require.NotZero(t, len(lines), "Expected some lines to parse from .star file, found none")
 	startIdx := -1
 	endIdx := len(lines)
@@ -3370,7 +3368,7 @@ func parseErrorMessage(t *testing.T, lines []string, header string) string {
 }
 
 func testLoadFunc(module string, logger telegraf.Logger) (starlark.StringDict, error) {
-	result, err := loadFunc(module, logger)
+	result, err := common.LoadFunc(module, logger)
 	if err != nil {
 		return nil, err
 	}
@@ -3384,6 +3382,35 @@ func testLoadFunc(module string, logger telegraf.Logger) (starlark.StringDict, e
 	return result, nil
 }
 
-func testNow(thread *starlark.Thread, _ *starlark.Builtin, args starlark.Tuple, kwargs []starlark.Tuple) (starlark.Value, error) {
+func testNow(_ *starlark.Thread, _ *starlark.Builtin, _ starlark.Tuple, _ []starlark.Tuple) (starlark.Value, error) {
 	return starlarktime.Time(time.Date(2021, 4, 15, 12, 0, 0, 999, time.UTC)), nil
 }
+
+func newStarlarkFromSource(source string) *Starlark {
+	return &Starlark{
+		Common: common.Common{
+			StarlarkLoadFunc: testLoadFunc,
+			Log:              testutil.Logger{},
+			Source:           source,
+		},
+	}
+}
+
+func newStarlarkFromScript(script string) *Starlark {
+	return &Starlark{
+		Common: common.Common{
+			StarlarkLoadFunc: testLoadFunc,
+			Log:              testutil.Logger{},
+			Script:           script,
+		},
+	}
+}
+
+func newStarlarkNoScript() *Starlark {
+	return &Starlark{
+		Common: common.Common{
+			StarlarkLoadFunc: testLoadFunc,
+			Log:              testutil.Logger{},
+		},
+	}
+}
